@@ -4,16 +4,20 @@ import os
 from dataclasses import dataclass, field
 from enum import auto
 from pathlib import Path
-from typing import Dict, List, Optional, Tuple
+from typing import Dict, List, Optional, Tuple, TypedDict, Union
 
 import pandas as pd
 import requests
-from deprecated import deprecated
 from SPARQLWrapper import JSON, SPARQLWrapper
 from strenum import StrEnum
 
-from cimsparql import url
-from cimsparql.url import Prefix, service, service_blazegraph
+from cimsparql.url import service, service_blazegraph
+
+
+class SparqlResult(TypedDict):
+    cols: List[str]
+    data: List[Dict[str, Dict[str, str]]]
+    out: List[Dict[str, str]]
 
 
 def data_row(cols: List[str], rows: List[Dict[str, str]]) -> Dict[str, str]:
@@ -50,15 +54,41 @@ def parse_namespaces_rdf4j(response: requests.Response) -> Dict[str, str]:
 
 @dataclass
 class ServiceConfig:
-    url: str
+    repo: str = field(default=os.getenv("GRAPHDB_REPO", "LATEST"))
+    protocol: str = "https"
+    server: str = field(default=os.getenv("GRAPHDB_SERVER", "127.0.0.1:7200"))
+    path: str = ""
     user: str = field(default=os.getenv("GRAPHDB_USER"))
     passwd: str = field(default=os.getenv("GRAPHDB_USER_PASSWD"))
     rest_api: RestApi = field(default=os.getenv("SPARQL_REST_API", RestApi.RDF4J))
 
+    # Parameters for rest api
+    # https://rdf4j.org/documentation/reference/rest-api/
+    distinct: bool = False
+    infer: bool = False
+    limit: Optional[int] = None
+    offset: Optional[int] = None
+    timout: Optional[int] = None
+
     def __post_init__(self):
         if self.rest_api not in RestApi:
             raise ValueError(f"rest_api must be one of {RestApi}")
 
+    @property
+    def url(self) -> str:
+        if self.rest_api == RestApi.BLAZEGRAPH:
+            return service_blazegraph(self.server, self.repo, self.protocol)
+        return service(self.repo, self.server, self.protocol, self.path)
+
+    @property
+    def parameters(self) -> Dict[str, Union[bool, int]]:
+        return {
+            "distinct": self.distinct,
+            "infer": self.infer,
+            "limit": self.limit,
+            "offset": self.offset,
+        }
+
 
 # Available formats from RDF4J API
 # https://rdf4j.org/documentation/reference/rest-api/
@@ -89,12 +119,7 @@ def require_rdf4j(f):
 
 
 class GraphDBClient:
-    def __init__(
-        self,
-        service: str,
-        infer: bool = False,
-        sameas: bool = True,
-    ) -> None:
+    def __init__(self, service_cfg: Optional[ServiceConfig] = None) -> None:
         """GraphDB client
 
         Args:
@@ -103,21 +128,30 @@ class GraphDBClient:
            infer: deduce further knowledge based on existing RDF data and a formal set of
            sameas: map same concepts from two or more datasets
         """
-        service = service or url.service(repo=os.getenv("GRAPHDB_REPO", "LATEST"))
-        self.service_cfg = ServiceConfig(url=service)
+        self.service_cfg = service_cfg or ServiceConfig()
         self.sparql = SPARQLWrapper(self.service_cfg.url)
         self.sparql.setReturnFormat(JSON)
         self.sparql.setCredentials(self.service_cfg.user, self.service_cfg.passwd)
-        self.sparql.addParameter("infer", str(infer))
-        self.sparql.addParameter("sameAs", str(sameas))
-
+        self._update_sparql_parameters()
         self._prefixes = None
 
+    def _update_sparql_parameters(self):
+        for key, value in self.service_cfg.parameters.items():
+            if value is not None:
+                self.set_parameter(key, str(value))
+
+    def set_parameter(self, key: str, value: str) -> None:
+        self.sparql.clearParameter(key)
+        self.sparql.addParameter(key, value)
+
+    def set_repo(self, repo: str) -> None:
+        self.service_cfg.repo = repo
+        self.sparql.endpoint = self.service_cfg.url
+
     @property
-    def prefixes(self) -> Prefix:
+    def prefixes(self) -> Dict[str, str]:
         if self._prefixes is None:
-            pref = self.get_prefixes()
-            self._prefixes = Prefix(pref)
+            self._prefixes = self.get_prefixes()
         return self._prefixes
 
     def update_prefixes(self, pref: Dict[str, str]):
@@ -129,29 +163,21 @@ class GraphDBClient:
     def __str__(self) -> str:
         return f"<GraphDBClient object, service: {self.service_cfg.url}>"
 
-    def query_with_header(self, query: str, limit: Optional[int] = None) -> str:
-        query = "\n".join([self.prefixes.header_str(query), query])
-        if limit is not None:
-            query += f" limit {limit}"
-        return query
-
-    def _exec_query(self, query: str, limit: Optional[int]):
-        self.sparql.setQuery(self.query_with_header(query, limit))
-
+    def _exec_query(self, query: str) -> SparqlResult:
+        self.sparql.setQuery(query)
+        self._update_sparql_parameters()
         processed_results = self.sparql.queryAndConvert()
 
         cols = processed_results["head"]["vars"]
         data = processed_results["results"]["bindings"]
         out = [{c: row.get(c, {}).get("value") for c in cols} for row in data]
-        return out, data_row(cols, data)
+        return {"out": out, "cols": cols, "data": data}
 
-    def exec_query(self, query: str, limit: Optional[int] = None) -> List[Dict[str, str]]:
-        out, _ = self._exec_query(query, limit)
-        return out
+    def exec_query(self, query: str) -> List[Dict[str, str]]:
+        out = self._exec_query(query)
+        return out["out"]
 
-    def get_table(
-        self, query: str, limit: Optional[int] = None
-    ) -> Tuple[pd.DataFrame, Dict[str, str]]:
+    def get_table(self, query: str) -> Tuple[pd.DataFrame, Dict[str, str]]:
         """
         Args:
            query: to sparql server
@@ -163,70 +189,83 @@ class GraphDBClient:
            >>> query = 'select * where { ?subject ?predicate ?object }'
            >>> gdbc.get_table(query, limit=10)
         """
-        out, data_row = self._exec_query(query, limit)
-        return pd.DataFrame(out), data_row
+        res = self._exec_query(query)
+        df = pd.DataFrame(res["out"]) if len(res["out"]) else pd.DataFrame(columns=res["cols"])
+        return df, data_row(res["cols"], res["data"])
 
     @property
     def empty(self) -> bool:
         """Identify empty GraphDB repo"""
         try:
-            self.get_table("SELECT * \n WHERE { ?s ?p ?o }", limit=1)
+            self.get_table("select * where {?s ?p ?o} limit 1")
             return False
         except IndexError:
             return True
 
     def get_prefixes(self) -> Dict[str, str]:
+        prefixes = default_namespaces()
+
         if self.service_cfg.rest_api == RestApi.BLAZEGRAPH:
             # Blazegraph does not expose prefixes over API
             # When using Blazegraph custom prefixes must be added via `update_prefixes`
             # By default we load a pre-defined set of prefixes
-            return default_namespaces()
+            return prefixes
 
-        prefixes = {}
         auth = requests.auth.HTTPBasicAuth(self.service_cfg.user, self.service_cfg.passwd)
         response = requests.get(self.service_cfg.url + "/namespaces", auth=auth)
         if response.ok:
-            prefixes = parse_namespaces_rdf4j(response)
-        else:
-            msg = (
-                "Could not fetch namespaces and prefixes from graphdb "
-                "Verify that user and password are correctly set in the "
-                "GRAPHDB_USER and GRAPHDB_USER_PASSWD environment variable"
-            )
-            raise RuntimeError(
-                f"{msg} Status code: {response.status_code} Reason: {response.reason}"
-            )
-        return prefixes
+            prefixes.update(parse_namespaces_rdf4j(response))
+            return prefixes
+        msg = (
+            "Could not fetch namespaces and prefixes from graphdb "
+            "Verify that user and password are correctly set in the "
+            "GRAPHDB_USER and GRAPHDB_USER_PASSWD environment variable"
+        )
+        raise RuntimeError(f"{msg} Status code: {response.status_code} Reason: {response.reason}")
 
-    def delete_repo(self):
+    def delete_repo(self) -> None:
         endpoint = delete_repo_endpoint(self.service_cfg)
         response = requests.delete(endpoint)
         response.raise_for_status()
 
-    def upload_rdf(self, fname: Path, rdf_format: str):
+    def upload_rdf(
+        self, fname: Path, rdf_format: str, params: Optional[Dict[str, str]] = None
+    ) -> None:
+        """
+        Upload data in RDF format to a srevice
+
+        Args
+            fname: Filename to the RDF file to upload
+            rdf_format: RDF file type (e.g. rdf/xml, rdf/json etc. Consult the RDF4J rest api
+                doc for a complete list of available options)
+            params: Additional parameters passed to the post request
+        """
         with open(fname, "rb") as infile:
             xml_content = infile.read()
 
         response = requests.post(
             self.service_cfg.url + UPLOAD_END_POINT[self.service_cfg.rest_api],
             data=xml_content,
+            params=params,
             headers={"Content-Type": MIME_TYPE_RDF_FORMATS[rdf_format]},
         )
         response.raise_for_status()
 
-    def update_query(self, query: str):
+    def update_query(self, query: str) -> None:
         """
         Function that passes a query via a post API call
         """
+        auth = requests.auth.HTTPBasicAuth(self.service_cfg.user, self.service_cfg.passwd)
         response = requests.post(
             self.service_cfg.url + UPLOAD_END_POINT[self.service_cfg.rest_api],
             data={"update": query},
             headers={"Content-Type": "application/x-www-form-urlencoded"},
+            auth=auth,
         )
         response.raise_for_status()
 
     @require_rdf4j
-    def set_namespace(self, prefix: str, value: str):
+    def set_namespace(self, prefix: str, value: str) -> None:
         auth = requests.auth.HTTPBasicAuth(self.service_cfg.user, self.service_cfg.passwd)
         response = requests.put(
             self.service_cfg.url + f"/namespaces/{prefix}",
@@ -244,18 +283,6 @@ class GraphDBClient:
         return response.text
 
 
-@deprecated(version="1.11", reason="Use cimsparqel.model.get_cim_model instead")
-def get_graphdb_client(
-    server: str,
-    graphdb_repo: str,
-    graphdb_path: str = "services/pgm/equipment/",
-    protocol: str = "https",
-):
-    from cimsparql.model import get_cim_model
-
-    return get_cim_model(server, graphdb_repo, graphdb_path, protocol)
-
-
 @dataclass
 class RepoInfo:
     uri: str
@@ -265,23 +292,27 @@ class RepoInfo:
     writable: bool
 
 
-def repos(server: str) -> List[RepoInfo]:
+def repos(service_cfg: Optional[ServiceConfig] = None) -> List[RepoInfo]:
     """
     List available repositories
     """
-    response = requests.get(server + "/repositories")
+
+    service_cfg = service_cfg or ServiceConfig()
+    auth = requests.auth.HTTPBasicAuth(service_cfg.user, service_cfg.passwd)
+
+    url = f"{service_cfg.protocol}://{service_cfg.server}/repositories"
+    response = requests.get(url, auth=auth, headers={"Accept": "application/json"})
     response.raise_for_status()
 
-    infos = []
-    for line in response.text.split("\n")[1:]:
-        if not line:
-            continue
-        uri, repo_id, title, readable, writable = line.strip().split(",")
-        readable = readable == "true"
-        writable = writable == "true"
-        info = RepoInfo(uri=uri, repo_id=repo_id, title=title, readable=readable, writable=writable)
-        infos.append(info)
-    return infos
+    def _repo_info(repo):
+        uri = repo["uri"]["value"]
+        repo_id = repo["id"]["value"]
+        title = repo["title"]["value"]
+        readable = repo["readable"]["value"] == "true"
+        writable = repo["writable"]["value"] == "true"
+        return RepoInfo(uri, repo_id, title, readable, writable)
+
+    return [_repo_info(repo) for repo in response.json()["results"]["bindings"]]
 
 
 def new_repo(
@@ -304,7 +335,7 @@ def new_repo(
     if response.status_code not in ignored_errors:
         response.raise_for_status()
 
-    return GraphDBClient(url)
+    return GraphDBClient(ServiceConfig(repo, protocol, server))
 
 
 def new_repo_blazegraph(url: str, repo: str, protocol: str = "https") -> GraphDBClient:
@@ -312,11 +343,10 @@ def new_repo_blazegraph(url: str, repo: str, protocol: str = "https") -> GraphDB
     config = config_bytes_from_template(template, {"repo": repo})
 
     response = requests.post(
-        f"http://{url}", data=config, headers={"Content-type": "application/xml"}
+        f"{protocol}://{url}", data=config, headers={"Content-type": "application/xml"}
     )
     response.raise_for_status()
-    client = GraphDBClient(service_blazegraph(url, repo, protocol))
-    client.service_cfg.rest_api = RestApi.BLAZEGRAPH
+    client = GraphDBClient(ServiceConfig(repo, protocol, url, rest_api=RestApi.BLAZEGRAPH))
     return client
 
 
