@@ -1,14 +1,15 @@
 from __future__ import annotations
 
 import warnings
+from contextlib import contextmanager
 from datetime import datetime
 from decimal import Decimal
-from typing import Any, Callable, Dict, List, Optional, Union
+from typing import Any, Callable, Dict, Generator, Optional, Union
 
 import pandas as pd
 
 from cimsparql.graphdb import GraphDBClient
-from cimsparql.query_support import combine_statements, unionize
+from cimsparql.templates import TYPE_MAPPER_QUERY
 
 # Type-caster that can be used with pandas. It can be python types, numpy dtypes or string
 # value. Examples: float, "int", int, "string"
@@ -32,10 +33,6 @@ def to_timedelta(duration: str) -> datetime.timedelta:
     return pd.to_timedelta(duration)
 
 
-def identity(x):
-    return x
-
-
 XSD_TYPE_MAP = {
     # Primitive types (https://www.w3.org/TR/xmlschema11-2/#built-in-primitive-datatypes)
     "boolean": lambda x: x.lower() in {"true", "1"},
@@ -64,6 +61,16 @@ sparql_type_map = {
 }
 
 
+@contextmanager
+def enforce_no_limit(client: GraphDBClient) -> Generator[GraphDBClient, None, None]:
+    orig_limit = client.service_cfg.limit
+    client.service_cfg.limit = None
+    try:
+        yield client
+    finally:
+        client.service_cfg.limit = orig_limit
+
+
 def build_type_map(prefixes: Dict[PREFIX, URI]) -> Dict[SPARQL_TYPE, TYPE_CASTER]:
     short_map = {"xsd": XSD_TYPE_MAP, "cim": CIM_TYPE_MAP}
 
@@ -77,58 +84,12 @@ def build_type_map(prefixes: Dict[PREFIX, URI]) -> Dict[SPARQL_TYPE, TYPE_CASTER
     return type_map
 
 
-class TypeMapperQueries:
-    def __init__(self, prefixes: Dict[PREFIX, URI]):
-        self.prefixes = prefixes
-
-    @property
-    def type_queries(self) -> List[List[str]]:
-        queries = [
-            ["?sparql_type rdf:type rdfs:Datatype", "?sparql_type owl:equivalentClass ?range"],
-            ["?sparql_type owl:equivalentClass ?range"],
-        ]
-
-        if "cims" in self.prefixes:
-            queries += self.cims_queries
-        return queries
-
-    @property
-    def cims_queries(self) -> List[List[str]]:
-        """
-        Return queries that require that cims namespace. We must distinguish between types
-        that is of type 'Primitive' and types that are of type 'CIMdatatype'. In the latter case
-        we must find the type of the 'value' attribute
-        """
-        return [
-            [
-                "?datatypevalue rdfs:domain ?sparql_type",
-                "?datatypevalue cims:dataType ?range",
-                '?range cims:stereotype "Primitive"',
-            ],
-            [
-                "?datatypevalue rdfs:domain ?sparql_type",
-                "?datatypevalue cims:dataType ?CIMdtype",
-                '?CIMdtype cims:stereotype "CIMDatatype"',
-                "?CIMdtypeValue rdfs:domain ?CIMdtype",
-                '?CIMdtypeValue rdfs:label "value"@en',
-                "?CIMdtypeValue cims:dataType ?range",
-            ],
-        ]
-
-    @property
-    def query(self) -> str:
-        select_query = "SELECT ?sparql_type ?range"
-        grouped = [combine_statements(*g, split=".\n") for g in self.type_queries]
-        union = unionize(*grouped)
-        return f"{select_query}\nWHERE\n{{\n{union}\n}}"
-
-
 class TypeMapper:
     def __init__(
         self, client: GraphDBClient, custom_additions: Optional[Dict[str, Any]] = None
     ) -> None:
-        self.prefixes = client.prefixes.prefixes
-        self.queries = TypeMapperQueries(self.prefixes)
+        self.prefixes = client.prefixes
+        self.query = TYPE_MAPPER_QUERY.substitute(self.prefixes)
         custom_additions = custom_additions or {}
         self.prim_type_map = build_type_map(self.prefixes)
         self.map = sparql_type_map | self.get_map(client) | self.prim_type_map | custom_additions
@@ -151,7 +112,9 @@ class TypeMapper:
             sparql-type -> python type map
 
         """
-        df = client.get_table(self.queries.query)[0]
+
+        with enforce_no_limit(client) as c:
+            df = c.get_table(self.query)[0]
         if df.empty:
             return {}
         return self.type_map(df)
@@ -189,8 +152,8 @@ class TypeMapper:
     def map_data_types(
         self, df: pd.DataFrame, col_map: Dict[COL_NAME, SPARQL_TYPE]
     ) -> pd.DataFrame:
-        """Maps the dtypes of a DataFrame to the python-corresponding types of the sparql-types from the
-        source data
+        """Maps the dtypes of a DataFrame to the python-corresponding types of the sparql-types from
+        the source data
 
         Args:
             df: DataFrame with columns to be converted
@@ -199,6 +162,8 @@ class TypeMapper:
             mapped DataFrame
 
         """
+        if df.empty:
+            return df
         type_caster = self.build_type_caster(col_map)
         df = map_base_types(df, type_caster)
         df = map_exceptions(df, type_caster)
